// Delcom C# USB StartUp Project Code 
// Jan 14, 2009
// This code includes the basic code to open the USB HID device.
// This code can be used a startup project for new projects

// Steps to make this project.
// 1 Create your project
// 2 Copy the follow file in the project source directory
//      DelcomHID.cs, DeviceManagement.cs, DeviceManademenedDeclarations.cs
//      FileIODeclarations.cs, Hid.cs, HidDeclarations.cs
// 3 Add the above file to your project. Use add existing in Solution Explorer
// 4 Add the following inside your namespace:
//      DelcomHID Delcom = new DelcomHID();   // declare the Delcom class

 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using HIDIOWINCS;   // Add are Delcom USB HID class

namespace HIDSTARTUPCS
{
    

    public partial class Form1 : Form
    {
        private DelcomHID Delcom = new DelcomHID();   // declare the Delcom class
        private DelcomHID.HidTxPacketStruct TxCmd;
        private DelcomHID.HidRxPacketStruct RxCmd;
        private byte[] myByte = new byte[16];
        private byte myLength;
        private UInt32 I2CError, DeviceError;
        private UInt32 DeviceVersion=0;


        //------------------------------------------------------------ //
        // The constructor class - Called in init
        //------------------------------------------------------------ //
        public Form1()
        {
            InitializeComponent();
            
        }


        //------------------------------------------------------------ //
        // Close the program
        //------------------------------------------------------------ //
        private void button_Quit_Click(object sender, EventArgs e)
        {
            Close();
        }

        //------------------------------------------------------------ //
        // Find and open the USB HID device
        //------------------------------------------------------------ //
        private void button_Open_Click(object sender, EventArgs e)
        {
            // Current TID and SID are not supported
            timer1.Stop();
            if (Delcom.Open() == 0)
            {
                UInt32 SerialNumber, Date, Month, Year;
                SerialNumber = DeviceVersion = Date = Month = Year = 0;
                Delcom.GetDeviceInfo(ref SerialNumber, ref DeviceVersion, ref Date, ref Month, ref Year);
                Year += 2000;
                label_DeviceName.Text = "DeviceName: " + Delcom.GetDeviceName();
                label_DeviceStatus.Text = "Device Status: Found. SerialNumber=" + SerialNumber.ToString() + " Version=" + DeviceVersion.ToString() + " " + Month.ToString() + "/" + Date.ToString() + "/" + Year.ToString();

                // always dump the first error
                Delcom.GetError(ref DeviceError);
                
     
            }
            else
            {
                label_DeviceName.Text = "DeviceName: offine";
                label_DeviceStatus.Text = "Error: Unable to open device.";
            }
            timer1.Start();
        }

        //------------------------------------------------------------ //
        // Close the USB HID device
        //------------------------------------------------------------ //
        private void button_Close_Click(object sender, EventArgs e)
        {
            Delcom.Close();
            label_DeviceName.Text = "DeviceName: offine";
            label_DeviceStatus.Text = "Device Closed.";
        }

        //------------------------------------------------------------ //
        // The timer fucntion
        //------------------------------------------------------------ //
        private void timer1_Tick(object sender, EventArgs e)
        {
            if (Delcom.IsOpen())
            {
                //sysbeep.Beep(2000, 50);
                timer1.Stop();
                DeviceError = 0;
                Delcom.GetError(ref DeviceError);
                if (DeviceError > 0) DisplayError(DeviceError);
                timer1.Start();

            }
        }

        //------------------------------------------------------------ //
        // DisplayError - Display any error in the message box
        //------------------------------------------------------------ //
        private void DisplayError(UInt32 DelcomError)
        {
            if (DelcomError>0)
            {
                String Msg;
                Msg = "USB Device Error Found. " + DelcomError.ToString() + "\r\n\r\n";
                Msg += "The following errors where found:\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0001)) Msg += "-Power On Bit Set.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0002)) Msg += "-Watch Dog Reset.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0004)) Msg += "-Low Voltage Reset.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0008)) Msg += "-Invaild ISR Reset.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0010)) Msg += "-USB No Data Error.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0100)) Msg += "-Write command failed or not supported.\r\n";
                if (Convert.ToBoolean(DelcomError & 0x0200)) Msg += "-Read command failed or not supported.\r\n";

                MessageBox.Show(Msg, "Warning", MessageBoxButtons.OK);

            }

            

        }

        //------------------------------------------------------------ //
        // Check is device is present and openned
        //------------------------------------------------------------ //
        private Boolean IsPresent()
        {
            if (Delcom.IsOpen()) return true;
            MessageBox.Show("Device not openned. Open device first!\r\nCommand canncelled.", "Warning - Device Not Openned!");
            return false;
        }

        //------------------------------------------------------------ //
        // UpdateI2CError - Read the I2C error bit and check the dialog
        // I2C check error box according. This function call after all
        // I2C functions to check that the I2C function suceeded.
        //------------------------------------------------------------ //
        private void UpdateI2CError()
        {
            if (Delcom.IsOpen()) {
                timer1.Stop();
                Delcom.ReadI2CError(ref I2CError);
                if (I2CError > 0)
                {
                    checkBox_I2CError.Checked = true;
                    if (checkBox_BeepOnError.Checked == true)
                        sysbeep.Beep(200, 100);
                }
                else checkBox_I2CError.Checked = false;
	            timer1.Start();
            }

        }

        //------------------------------------------------------------ //
        // Write up to 8 byte to the I2C port
        //------------------------------------------------------------ //
        private void button_Write_Click(object sender, EventArgs e)
        {


            if (!IsPresent()) return;
            try
            {
                myByte[0] = Convert.ToByte(textBox_WrByte0.Text);
                myByte[1] = Convert.ToByte(textBox_WrByte1.Text);
                myByte[2] = Convert.ToByte(textBox_WrByte2.Text);
                myByte[3] = Convert.ToByte(textBox_WrByte3.Text);
                myByte[4] = Convert.ToByte(textBox_WrByte4.Text);
                myByte[5] = Convert.ToByte(textBox_WrByte5.Text);
                myByte[6] = Convert.ToByte(textBox_WrByte6.Text);
                myByte[7] = Convert.ToByte(textBox_WrByte7.Text);
                myByte[8] = Convert.ToByte(textBox_WrByte8.Text);
                myLength = Convert.ToByte(textBox_WrLength.Text);
                
            }
            catch (Exception ex)
            {
                MessageBox.Show("I2C Write parameters out of range or invalid!", "Warning - Range Error!");
                return; //exit
            }

            timer1.Stop();
            // Now fill the USB data sturture and send it on it's way
            TxCmd.MajorCmd = 102;
            TxCmd.MinorCmd = 60;
            TxCmd.LSBData = myByte[0];
            TxCmd.MSBData = myLength;
            TxCmd.ExtData0 = myByte[1];
            TxCmd.ExtData1 = myByte[2];
            TxCmd.ExtData2 = myByte[3];
            TxCmd.ExtData3 = myByte[4];
            TxCmd.ExtData4 = myByte[5];
            TxCmd.ExtData5 = myByte[6];
            TxCmd.ExtData6 = myByte[7];
            TxCmd.ExtData7 = myByte[8];
            
            Delcom.SendCommand(TxCmd); // Send the command
            timer1.Start();
            UpdateI2CError();
        }

        //------------------------------------------------------------ //
        // I2C Read Function 
        //------------------------------------------------------------ //
        private void button_Read_Click(object sender, EventArgs e)
        {
            if (!IsPresent()) return;
            try
            {
                myByte[0] = Convert.ToByte(textBox_RdByte0.Text);
                myLength = Convert.ToByte(textBox_ReadLength.Text);

            }
            catch (Exception ex)
            {
                MessageBox.Show("I2C Read parameters out of range or invalid!", "Warning - Range Error!");
                return; //exit
            }

            timer1.Stop();
            // Now fill the USB data sturture and send it on it's way
            TxCmd.MajorCmd = 101;
            TxCmd.MinorCmd = 63;
            TxCmd.HidData0 = myByte[0];
            TxCmd.MSBData = myLength;
            
            Delcom.SendCommand(TxCmd); // Send the command


            RxCmd.Data0 = 60;
            Delcom.SendRdCommand(ref RxCmd, 8); // Send the read command
            textBox_RdByte1.Text = RxCmd.Data0.ToString();
            textBox_RdByte2.Text = RxCmd.Data1.ToString();
            textBox_RdByte3.Text = RxCmd.Data2.ToString();
            textBox_RdByte4.Text = RxCmd.Data3.ToString();
            textBox_RdByte5.Text = RxCmd.Data4.ToString();
            textBox_RdByte6.Text = RxCmd.Data5.ToString();
            textBox_RdByte7.Text = RxCmd.Data6.ToString();
            textBox_RdByte8.Text = RxCmd.Data7.ToString();

            timer1.Start();
            UpdateI2CError();
        }
    
        //------------------------------------------------------------ //
        // I2C Sel 8 Read Function 
        //------------------------------------------------------------ //
        private void button_Sel8Read_Click(object sender, EventArgs e)
        {
            if (!IsPresent()) return;
            try
            {
                myByte[0] = Convert.ToByte(textBox_Sel8RdByte0.Text);
                myByte[1] = Convert.ToByte(textBox_Sel8RdByte1.Text);
                myByte[2] = Convert.ToByte(textBox_Sel8RdByte2.Text);
                myLength = Convert.ToByte(textBox_Sel8RdLength.Text);

            }
            catch (Exception ex)
            {
                MessageBox.Show("I2C Sel8 read parameters out of range or invalid!", "Warning - Range Error!");
                return; //exit
            }

            timer1.Stop();
            // Now fill the USB data sturture and send it on it's way
            TxCmd.MajorCmd = 101;
            TxCmd.MinorCmd = 63;
            TxCmd.HidData0 = myByte[0];
            TxCmd.HidData1 = myByte[1];
            TxCmd.HidData3 = myByte[2];
            TxCmd.MSBData = myLength;
            
            Delcom.SendCommand(TxCmd); // Send the command

            RxCmd.Data0 = 61;
            Delcom.SendRdCommand(ref RxCmd, 8); // Send the read command
            textBox_Sel8RdByte3.Text = RxCmd.Data0.ToString();
            textBox_Sel8RdByte4.Text = RxCmd.Data1.ToString();
            textBox_Sel8RdByte5.Text = RxCmd.Data2.ToString();
            textBox_Sel8RdByte6.Text = RxCmd.Data3.ToString();
            textBox_Sel8RdByte7.Text = RxCmd.Data4.ToString();
            textBox_Sel8RdByte8.Text = RxCmd.Data5.ToString();
            textBox_Sel8RdByte9.Text = RxCmd.Data6.ToString();
            textBox_Sel8RdByte10.Text = RxCmd.Data7.ToString();

            timer1.Start();
            UpdateI2CError();
        }

        //------------------------------------------------------------ //
        // I2C Sel 16 Read Function 
        //------------------------------------------------------------ //
        private void button_Sel16Read_Click(object sender, EventArgs e)
        {
            if (!IsPresent()) return;
            try
            {
                myByte[0] = Convert.ToByte(textBox_Sel16RdByte0.Text);
                myByte[1] = Convert.ToByte(textBox_Sel16RdByte1.Text);
                myByte[2] = Convert.ToByte(textBox_Sel16RdByte2.Text);
                myByte[3] = Convert.ToByte(textBox_Sel16RdByte3.Text);
                myLength = Convert.ToByte(textBox_Sel16RdLength.Text);

            }
            catch (Exception ex)
            {
                MessageBox.Show("I2C Sel16 Read parameters out of range or invalid!", "Warning - Range Error!");
                return; //exit
            }

            timer1.Stop();
            // Now fill the USB data sturture and send it on it's way
            TxCmd.MajorCmd = 101;
            TxCmd.MinorCmd = 63;
            TxCmd.HidData0 = myByte[0];
            TxCmd.HidData1 = myByte[1];
            TxCmd.HidData2 = myByte[2];
            TxCmd.HidData3 = myByte[3];
            TxCmd.MSBData = myLength;
            
            Delcom.SendCommand(TxCmd); // Send the command

            RxCmd.Data0 = 62;
            Delcom.SendRdCommand(ref RxCmd, 8); // Send the read command
            textBox_Sel16RdByte4.Text = RxCmd.Data0.ToString();
            textBox_Sel16RdByte5.Text = RxCmd.Data1.ToString();
            textBox_Sel16RdByte6.Text = RxCmd.Data2.ToString();
            textBox_Sel16RdByte7.Text = RxCmd.Data3.ToString();
            textBox_Sel16RdByte8.Text = RxCmd.Data4.ToString();
            textBox_Sel16RdByte9.Text = RxCmd.Data5.ToString();
            textBox_Sel16RdByte10.Text = RxCmd.Data6.ToString();
            textBox_Sel16RdByte11.Text = RxCmd.Data7.ToString();

            timer1.Start();
            UpdateI2CError();
        }

        //------------------------------------------------------------ //
        // I2C Output Type
        //------------------------------------------------------------ //
        private void checkBox_OutputType_CheckedChanged(object sender, EventArgs e)
        {
            if (DeviceVersion < 23)
            {
                MessageBox.Show("This option is not supported on this firmware version.\r\nYou must have firmware version of 23 or greater.", "Unsupported feature", MessageBoxButtons.OK);
                return;
            }

            timer1.Stop();
            // Now fill the USB data sturture and send it on it's way
            TxCmd.MajorCmd = 101;
            TxCmd.MinorCmd = 110;
            TxCmd.LSBData = 0;
            TxCmd.MSBData = 0;

            if(checkBox_OutputType.Checked == true)     TxCmd.MSBData = 0x01;
            else                                        TxCmd.LSBData = 0x01;
            
            Delcom.SendCommand(TxCmd); // Send the command
            timer1.Start();
        }

      

    
       

    }

    // External declaration to the windows beep function.
    public class sysbeep
    {
        [DllImport("Kernel32.dll")]
        public static extern bool Beep(UInt32 frequency, UInt32 duration);
    }

}